/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.sinfo.common;

import java.util.HashSet;
import java.util.Set;

import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.common.AbstractCalculationResult;

/**
 * Processor to generate the facet and data series of infrastructure flood durations
 *
 * @author Matthias Schäfer
 *
 */
public final class FloodDurationProcessor extends AbstractSInfoLineProcessor<AbstractCalculationResult> {

    public static final String FACET_FLOOD_DURATION = "sinfo_facet_flood_duration";

    public static final String FACET_FLOOD_DURATION_DESCRIPTION = "sinfo_facet_flood_duration.description";

    public static final String FACET_MAIN_VALUE_1_DURATION = "mainvalue.1.duration";

    public static final String FACET_MAIN_VALUE_DURATION_DESCRIPTION = "mainvalue.duration.description";

    private static final String I18N_AXIS_LABEL = "sinfo.chart.flood_duration.section.yaxis.label";

    private static final Set<String> HANDLED_FACET_TYPES = new HashSet<>();

    static {
        HANDLED_FACET_TYPES.add(FACET_FLOOD_DURATION);
        HANDLED_FACET_TYPES.add(FACET_MAIN_VALUE_1_DURATION);
    }

    public FloodDurationProcessor() {
        super(I18N_AXIS_LABEL, HANDLED_FACET_TYPES);
    }


    @Override
    protected double[][] doGetPoints(final AbstractCalculationResult data, final String facetName) {
        if (FACET_FLOOD_DURATION.contentEquals(facetName))
            return data.getStationPoints(SInfoResultType.floodDuration);

        if (FACET_MAIN_VALUE_1_DURATION.contentEquals(facetName))
            return data.getStationPoints(SInfoResultType.mainValue1Duration);

        final String error = String.format("Unknown facet name: %s", facetName);
        throw new UnsupportedOperationException(error);
    }

    public static Facet createFloodDurationFacet(final CallContext context, final String hash, final String id,
            final AbstractCalculationResult result, final int index) {
        return AbstractSInfoLineProcessor.createFacet(context, hash, id, result, index, I18N_AXIS_LABEL,
                FACET_MAIN_VALUE_1_DURATION, FACET_MAIN_VALUE_DURATION_DESCRIPTION);
    }

    public static Facet createMainValueDurationFacet(final CallContext context, final String hash, final String id,
            final AbstractCalculationResult result, final int index) {
        return AbstractSInfoLineProcessor.createFacet(context, hash, id, result, index, I18N_AXIS_LABEL,
                FACET_FLOOD_DURATION, FACET_FLOOD_DURATION_DESCRIPTION);
    }
}